<?php
/*
 * User-friendly interface to SIEVE server-side mail filtering.
 * Plugin for Squirrelmail 1.2+
 *
 * Copyright (c) 2002 Alexandros Vellis <avel@users.sourceforge.net>
 *
 * Licensed under the GNU GPL. For full terms see the file COPYING that came
 * with the Squirrelmail distribution.
 *
 */

/*	makesinglerule: Get a $sess[] array and build _part of_
 *			a SIEVE rule.
 *			$rule = a rule array
 *			$type = text|rule
 *			$default = 0 or 1
 *
 *	makesieverule:	Get an array of sess['blah'] arrays and build
 *			our rule.
 *
 *	It's all kludgy.
 */

require_once("config.php");

function makesinglerule($rule, $type="rule") {

/* Arguments are as follows:

	$rule	just a rule array.
	
	$type	text = return a textual description of the rule.
		rule = return a string with the appropriate SIEVE code.
	
	$default boolean - is this a default rule ("else") or a normal rule
		 (if.... elseif...)
*/

global $maxitems;

/* Step zero :-) : serialize & encode my array */

$coded = urlencode(base64_encode(serialize($rule)));

$out = "#START_SIEVE_RULE".$coded."END_SIEVE_RULE\n";

/* Step one: make the if clause */

/* The actual 'if' will be added by makesieverule() */

if($rule['type']=="4") {
 	$text .= _("For <strong>ALL</strong> incoming messages; ");
} else {
	$text .= "<strong>"._("If")."</strong> ";
} 

switch ($rule['type']) {
case "1":	/* address --- deprecated */
	for ( $i=0; $i<3; $i++) {
		$out .= 'address :'.${'address'.$i};
		if(${'addressrel'.$i} != "0") {
			$out .= ":";
		}
	}
	break;

case "2":	/* header */
	switch ($rule['condition']) {
		case "or":
			$out .= "anyof (";
			$text .= _("<em>any</em> of the following mail headers match: ");
			break;
		case "and":
			$out .= "allof (";
			$text .= _("<em>all</em> of the following mail headers match: ");
			break;
		default: /* condition was not defined, so there's only one header item. */
			$lonely = true;
			break;
	}
	/* if ( $i<sizeof($rule['headermatch'][$i] > 1) {
		$out .=
	}
	*/
	for ( $i=0; $i<sizeof($rule['headermatch']); $i++) {
		$text .= _("the header");
		$text .= " &quot;".$rule['header'][$i]."&quot; ";
		
 		switch ($rule['matchtype'][$i]) {
 			case "is":
 				$out .= "header :is";
				$text .= _("is");
 				break 1;
 			case "is not":
 				$out .= "not header :is";
				$text .= _("is not");
 				break 1;
 			case "contains":
 				$out .= "header :contains";
				$text .= _("contains");
 				break 1;
 			case "does not contain":
 				$out .= "not header :contains";
				$text .= _("does not contain");
 				break 1;
 			default:
 				break 1;
 		}
		$out .= " \"".$rule['header'][$i]."\" ";
		$out .= "\"".$rule['headermatch'][$i]."\"";
		
		$text .= " &quot;".$rule['headermatch'][$i]."&quot;";

		if($rule['headermatch'][$i+1]) {
			$out .= ",\n";
			$text .= ", ";
		} elseif($lonely) {
			$out .= "\n";
			$text .= ", ";
		} else {
			$out .= ")\n";
			$text .= "; ";
		}

	}
	
	/* if ($rule['headerrel0'] == "0") {
		$text .= " \"".$rule['header0']."\" ".$rule['matchtype0']." \"".$rule['headermatch0']."\", ";
	} else {
		for ($i=0; $i<sizeof($rule['headermatch']); $i++) {
			$text .= _("the header");
			$text .=' &quot;'.$rule['header'][$i].'" '.$rule['matchtype'][$i].' "'.$rule['headermatch'][$i].'&quot;, ';
			if ($rule['headerrel'][$i]=="0") {
				break;
			} 
		}		
	} */
	break;

case "3":	/* size */
	$out .= 'size :';
	$text .= _("the size of the message is");
	$text .= "<em>";
	
	if($rule['sizerel'] == "bigger") {
		$out .= "over ";
		$text .= _(" bigger");
	} else {
		$out .= "under ";
		$text .= _(" smaller");
	}
	$text .= " "._("than")." ".$rule['sizeamount']." ".$rule['sizeunit']."</em>, ";
	
	$out .= $rule['sizeamount'];
	
	if($rule['sizeunit']=="kb") {
		$out .= "K\n";
	} elseif($rule['sizeunit']=="mb") {
		$out .= "M\n";
	}
	break;

case "4":	/* always */
	$out .= "true\n";
	break;
}



/* step two: make the then clause */

$out .= "{\n";

if($rule['type']!="4") {
	$text .= "<strong>";
	$text .= _("then");
	$text .= "</strong> ";
}

switch ($rule['action']) {
case "1":	/* keep (default) */
	$out .= "keep;";
	$text .= _("<em>keep</em> it.");
	break;

case "2":	/* discard */
	$out .= "discard;";
	$text .= _("<em>discard</em> it.");
	break;

case "3":	/* reject w/ excuse */
	
	$out .= "reject text:\n".$rule['excuse']."\n.\n;";
	$text .= _("<em>reject</em> it, sending this excuse back to the sender:")." \"".$rule['excuse']."\".";
	break;

case "4":	/* redirect to address */
	$out .= "redirect \"".$rule['redirectemail']."\";";
	$text .= _("<em>redirect</em> it to the email address")." ".$rule['redirectemail'].".";
	break;

case "5":	/* fileinto folder */

	$out .= 'fileinto "'.$rule['folder'].'";';
	$text .= _("<em>file</em> it into the folder <strong>").$rule['folder']."</strong>.";
	break;
case "6":      /* vacation message */
	$addresses = str_replace(",",'","',str_replace(" ","",$rule['vac_addresses']));
 	$out .= 'vacation :days '.$rule['vac_days'].' :addresses ["'.$addresses.
 	'"] "'.$rule['vac_message'].'";';
 	$text .= _("reply with this vacation message: ").$rule['vac_message'];
 	break;
default:
	return "0";
	break;
}

if ($rule[stop] != '') {
	$text .= _(" Then <strong>STOP</strong> processing rules.");
	$out .= "\nstop;";
}

$out .= "\n}";

if ($type == "text") {
	return $text;
} else {
	return $out;
}

}


/* -------------------------------------------------- */


/* Make a complete set of rules, i.e. a SIEVE script */
/* $rulearray is an array of associative arrays describing a rule each. */
/* $defaultrulearray is an array that should have only one element,
 * $defaultrulearray[0], of the default rule */

function makesieverule ($rulearray) {

	if ( (sizeof($rulearray) == 0) || $rulearray[0] == "0" ) {
		return false;
	}

 	$out = "require [\"fileinto\", \"reject\", \"vacation\"];\n"; 

	for ($i=0; $i<sizeof($rulearray); $i++) {
		switch ($i) {
			case 0:				$out .= "if\n";		break;
			/* case sizeof($rulearray): 	$out .= "\nelse\n";	break; */
 			default:			$out .= "\nif\n";	break;
		}		
		$out .= makesinglerule($rulearray[$i],"rule");
	}
	
	/*
	print "The script is: <pre>";
	print $out;
	print "</pre>"; 
	*/
	return $out;
}

?>
