<?php
/*
 * User-friendly interface to SIEVE server-side mail filtering.
 * Plugin for Squirrelmail 1.2+
 *
 * Copyright (c) 2002 Alexandros Vellis <avel@users.sourceforge.net>
 *
 * Based on Dan Ellis' test scripts that came with sieve-php.lib
 * <danellis@rushmore.com> <URL:http://sieve-php.sourceforge.net>
 *
 * Licensed under the GNU GPL. For full terms see the file COPYING that came
 * with the Squirrelmail distribution.
 *
 */

/* table.php: main routine that shows a table of all the rules and allows
 * manipulation. */

chdir ("..");
require_once('../src/validate.php');
require_once('../functions/page_header.php');
require_once('../functions/i18n.php');
require_once('../functions/imap.php');
require_once('../functions/strings.php');
require_once('../src/load_prefs.php');

chdir ('avelsieve');
require_once "sieve-php.lib";
require_once "table_html.inc";
require_once "getrule.inc";
require_once "buildrule.inc";
require_once "config.php";
require_once "support.inc";
require_once "sieve.inc";

session_start();

/* Let's roll. */

$acctpass = OneTimePadDecrypt($key, $onetimepad);

/* $sieve=new sieve($server, $port, $acctname, $acctpass, $adminacct); */
$sieve=new sieve($imapServerAddress, $port, $username, $acctpass, $username);

if (!($conservative && $rules)) {
	/* If we are conservative on our connections
	to IMAP server AND the rules are cached, don't login! */ 

	if ($sieve->sieve_login()){	/* User has logged on */
		/* print "Welcome! Login successful.\n"; */
		/* execution will continue below... */
	} else {
		print "Could not login to timsieved daemon on your IMAP server ".$imapServerAddress.".";
		print "<br />Please contact your administrator.";
		exit;
	}
}

/* Here we continue normally. */

/* Get phpscript script. If it exists. */

if (!($conservative && $rules)) {

if($sieve->sieve_listscripts() /* && isset($currentrules) */ ){
	if(!isset($sieve->response)) {
		/* There is no SIEVE script on the server. */
		print_create_new();
		printfooter();
		exit;
		
	} elseif(is_array($sieve->response)){
		/* print "DEBUG: listscripts response is an array."; */
		$i = "1";
		foreach($sieve->response as $line){
			$scriptname[$i] = $line;
			$i++;
		}
		/* print $scriptname["1"]; -- here $scriptname["1"]=="phpscript" ... :-) */
	} else {
		
		print "DEBUG: listscripts returns a single thing? Strange. It doesnt do that usually.";
		
		if($response == "phpscript") {
			print_phpscript();
			if($sieve->sieve_getscript("phpscript")){
				if(is_array($sieve->response))
					foreach($sieve->response as $line){
						$sieverule .= $line;
					}
                    		} else {
					print $sieve->response;
				}
           	 
		} else {
			print "ERROR: Obviously you can write SIEVE scripts without this interface! :)";
			exit;
		}
	
	}
}

}
/* If we got here, everything should be ok.*/

/* On to the CODE THAT EXECUTES IF PHPSCRIPT EXISTS (or if a new rule has been created). */

if($logout) {
	/* Activate phpscript and log out. */
	
	if ($conservative) { 
		if (!($sieve->sieve_login())){
			print 'Error: Could not login to timsieved daemon on your IMAP
			server '.$imapServerAddress.' in order to save your rules.<br />
			Please contact your administrator.';
			exit;
		}
	}

	if ($newscript = makesieverule($rules)) {

		upload_script($newscript);

		if(!($sieve->sieve_setactivescript("phpscript"))){
			/* Just to be safe. */
			print 'Could not set active script on your IMAP server '.$imapServerAddress.'.<br />';
			print 'Please contact your administrator.';
			exit;
		}
		$sieve->sieve_logout();
		/* chdir(".."); */
	
	} else {

		upload_script(""); /* upload a null thingie!!! :-)
		This works for now... some time it will get better. */
		
		/* if(sizeof($rules) == "0") {
			delete_script();
		} */

	}
	
	session_unregister('rules');
	// $location = get_location();
	header("Location: ../../src/options.php\n\n");
	exit();

} elseif ($add) {
	/* code to start wizard to add a new script */
	header("Location: addrule.php");
	exit;
}

displayPageHeader($color, 'None');
//putenv("LANG=ru_RU");
//print setlocale(LC_ALL, "");
//print

/* Works: */
bindtextdomain ('avelsieve', '../../locale');
textdomain ('avelsieve');

print_my_header();

/*
print "DEBUG:<pre>";
print_r($HTTP_SESSION_VARS);
print "</pre>";
*/

if(!$returnnewrule) {
	$returnnewrule = "no";
}

/* Actually GET the script 'phpscript'. */

if (!($conservative && $rules)) {
	if($sieve->sieve_getscript("phpscript")){
		if(is_array($sieve->response))
			foreach($sieve->response as $line){
			/* DEBUG:  print "$line\n"; */
			$sieverule .= "$line\n";
		} else {
			print 'Error encountered while receiving your rules.
			Probably the script is size null. Please contact your administrator.';
			exit;
		}
	}
	/* $sieverule has a SIEVE script. Parse that. */
	$rules = getruledata($sieverule);
	if (($rules == "0" && $returnnewrule == "no") ) {
		/* NO SCRIPT, WHAT HAPPENS IF THERE IS NO SCRIPT. (Stop shouting). */
		/* if (($rules == "0" && $returnnewrule == FALSE) || (!is_array($rules)) ) { */
		/* print_header(); */
		print_create_new();
		print_buttons_new();
		print_footer();
		exit;
	} else {
		$defaultrule = getruledata($sieverule);
		session_register('rules');
		/* print "DEBUG: found the foolowing default rule: <pre>"; print_r($defaultrule); print "</pre>"; */
	}
	/* now, $rules is an array of associative arrays of rules. :-) */
} else {
	/* print "DEBUG: Conservative is true AND rules is true. CACHE MAGIC! :-p"; */

	/* Rather, *add* the script to the SIEVE server */
	/* $currentrules */
	/* 
	for($i=0; $i<sizeof($currentrules[0]); $i++) {
		$rules[$i] = $rulearray[$i] = unserialize(base64_decode(urldecode($currentrules[$i])));
	}
	*/
}

// print "<pre>"; print_r($rules); print "</pre>";

if ($rm) {

	for ($i=0; $i<sizeof($rules); $i++ ) {
		// print "Debug: rm [ $i ] is $rm[$i] <br />";
		if ($rm[$i]) {
			$rules = array_del($rules, $i);
			break;
		}
	}

	if(sizeof($rules) == "0") {
		// print "DEBUG: Ok, size of rules is 0 apparently.";

		if (!$conservative) {
			delete_script();
		}
		print '<p>';
		print _("All your rules have been deleted");
		print '</p>';
		session_unregister('rules');
		print_create_new();
		print_buttons_new();
		print_footer();
		exit;
	} else {
		// print "DEBUG: rules is not sized 0";

		if (!($conservative && $rules)) {
			$newscript = makesieverule($rules);
			upload_script($newscript);
		} else {
			session_register('rules');
		}
	} /* end else */

} elseif ($mvup) {

	for ($i=0; $i<sizeof($rules); $i++ ) {
		if ($mvup[$i]) {
			$rules = array_swapval($rules, $i, $i-1);
			break;
		}
	}
	
	if (!($conservative && $rules)) {
		$newscript = makesieverule($rules);
		upload_script($newscript);
	} else {
		session_register('rules');
	}

} elseif ($mvdn) {

	for ($i=0; $i<sizeof($rules); $i++ ) {
		if ($mvdn[$i]) {
			$rules = array_swapval($rules, $i, $i+1);
			break;
		}
	}
	if (!($conservative && $rules)) {
		$newscript = makesieverule($rules);
		upload_script($newscript);
	} else {
		session_register('rules');
	}
}	


if ($returnnewrule!="no") { /* Get the new rule and put it in the script */
	
	session_unregister('part');
	$newrule = unserialize(base64_decode(urldecode($returnnewrule)));
	$returnnewrule = "no";
	session_register('returnnewrule');
	
	if (!is_array($rules)) {
		unset($rules);
		$rules[0] = $newrule;
	} else {
		$rules[sizeof($rules)] = $newrule;
	}
	
	if ($conservative) {
		session_register('rules');
	} else {
		$newscript = makesieverule($rules);
		upload_script($newscript);

	} 
}

session_register('returnnewrule');

/* ----------------------------------------------------------------------------------- */

/* Printing, part one: the table with the rules. */

print_table_header();

$location = get_location();

for ($i=0; $i<sizeof($rules); $i++) {
	print "<tr><td>".($i+1)."</td><td>";
	print makesinglerule($rules[$i], "text");
	print '</td><td><p style="white-space: nowrap">';

	/* print '</td><td><input type="checkbox" name="rm'.$i.'" value="1" /></td></tr>'; */
	if($useimages) {
			print ' <input type="image" name="rm['.$i.']" title="'._('Delete').'"
			src="'.$location.'/images/'.$imagetheme.'/del.png" alt="'._('Delete').'" value="Delete" />';
	} else {
			print '<input type="submit" name="rm['.$i.']" value="'._('Delete').'" />';
	}		

	if ($i != 0) {
		if($useimages) {
			print ' <input type="image" name="mvup['.$i.']" title="'._('Move Up').'"
			src="'.$location.'/images/'.$imagetheme.'/up.png" alt="'._('Move Up').'" value="Move Up" />';
		} else {
			print '<input type="submit" name="mvdn['.$i.']" value="'._('Move Down').'" />';
		}

	}


	if ($i != sizeof($rules)-1 ) {
		if($useimages) {
			print ' <input type="image" name="mvdn['.$i.']" title="'._('Move Down').'"
			src="'.$location.'/images/'.$imagetheme.'/down.png" alt="'._('Move Down').'" value="Move Down" />';
		} else {
			print '<input type="submit" name="mvdn['.$i.']" value="'._('Move Down').'" />';
		}
	}

	print '</td></tr>';
}

print '
</table>
';

print_buttons();

print_footer();
textdomain('squirrelmail');

?>
